@@ -72,6 +72,7 @@ G_BEGIN_DECLS
 #define GST_IS_CURLHTTPSRC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_CURLHTTPSRC))
 
+#define GSTCURL_FUNCTIONTRACE 0
 #if GSTCURL_FUNCTIONTRACE
 #define GSTCURL_FUNCTION_ENTRY(x) GST_DEBUG_OBJECT(x, "Entering function");
 #define GSTCURL_FUNCTION_EXIT(x) GST_DEBUG_OBJECT(x, "Leaving function");
@@ -98,10 +99,17 @@ typedef enum
 #ifdef CURL_VERSION_HTTP2
     GSTCURL_HTTP_VERSION_2_0,
 #endif
-    GSTCURL_HTTP_NOT,           /* For future use, incase not HTTP protocol! */
+    GSTCURL_HTTP_NOT,           /* For future use if HTTP protocol not used! */
     GSTCURL_HTTP_VERSION_MAX
   } GstCurlHttpVersion;
 
+typedef enum _GstCGstCurlHttpSrcSeekable
+{
+    GSTCURL_SEEKABLE_UNKNOWN,
+    GSTCURL_SEEKABLE_TRUE,
+    GSTCURL_SEEKABLE_FALSE
+} GstCGstCurlHttpSrcSeekable;
+
 struct _GstCurlHttpSrcMultiTaskContext
 {
   GstTask     *task;
@@ -158,6 +166,8 @@ struct _GstCurlHttpSrc
   GstStructure *request_headers;  /* CURLOPT_HTTPHEADER */
   struct curl_slist *slist;
   gboolean accept_compressed_encodings; /* CURLOPT_ACCEPT_ENCODING */
+  gint64 request_position;     /* Seek to this position. */
+  gint64 stop_position;        /* Stop at this position. */
 
   /* Connection options */
   glong allow_3xx_redirect;     /* CURLOPT_FOLLOWLOCATION */
@@ -213,6 +223,8 @@ struct _GstCurlHttpSrc
   guint status_code;
   gchar *reason_phrase;
   gboolean hdrs_updated;
+  guint64 content_size;
+  GstCGstCurlHttpSrcSeekable seekable;
 
   CURLcode curl_result;
   char curl_errbuf[CURL_ERROR_SIZE];
