@@ -52,11 +52,25 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
   gboolean ret = FALSE;
+  GstVulkanInstance *instance = gst_vulkan_instance_new ();
+  gboolean have_instance = gst_vulkan_instance_open (instance, NULL);
+  const gchar *env_vars[] =
+      { "VK_ICD_FILENAMES", "VK_DRIVER_FILES", "VK_ADD_DRIVER_FILES", NULL };
+#ifndef G_OS_WIN32
+  const gchar *kernel_paths[] = { "/dev/dri", NULL };
+  const gchar *kernel_names[] = { "renderD", NULL };
+
+  /* features get updated upon changes in /dev/dri/renderD* */
+  gst_plugin_add_dependency (plugin, NULL, kernel_paths, kernel_names,
+      GST_PLUGIN_DEPENDENCY_FLAG_FILE_NAME_IS_PREFIX);
+
+  /* features get updated upon changes on VK_ICD_FILENAMES envvar */
+#endif
+  gst_plugin_add_dependency (plugin, env_vars, NULL, NULL,
+      GST_PLUGIN_DEPENDENCY_FLAG_NONE);
 
   ret |= GST_DEVICE_PROVIDER_REGISTER (vulkandeviceprovider, plugin);
 
-  ret |= GST_ELEMENT_REGISTER (vulkansink, plugin);
-
   ret |= GST_ELEMENT_REGISTER (vulkanupload, plugin);
 
   ret |= GST_ELEMENT_REGISTER (vulkandownload, plugin);
@@ -71,11 +85,24 @@ plugin_init (GstPlugin * plugin)
 
   ret |= GST_ELEMENT_REGISTER (vulkanoverlaycompositor, plugin);
 #endif
+  if (have_instance && instance->n_physical_devices) {
+    for (gint i = 0; i < instance->n_physical_devices; i++) {
+      GstVulkanDevice *device = gst_vulkan_device_new_with_index (instance, i);
 #if GST_VULKAN_HAVE_VIDEO_EXTENSIONS
-  GST_ELEMENT_REGISTER (vulkanh264dec, plugin);
-  GST_ELEMENT_REGISTER (vulkanh265dec, plugin);
+      if (gst_vulkan_device_is_extension_enabled (device,
+              VK_KHR_VIDEO_DECODE_H264_EXTENSION_NAME)) {
+        ret |= gst_vulkan_h264_decoder_register (plugin, device, GST_RANK_NONE);
+      }
+      if (gst_vulkan_device_is_extension_enabled (device,
+              VK_KHR_VIDEO_DECODE_H265_EXTENSION_NAME)) {
+        ret |= gst_vulkan_h265_decoder_register (plugin, device, GST_RANK_NONE);
+      }
 #endif
-
+      ret |= gst_vulkan_sink_register (plugin, device, GST_RANK_NONE);
+      gst_object_unref (device);
+    }
+  }
+  gst_object_unref (instance);
   return ret;
 }
 
