@@ -24,6 +24,7 @@
 
 #include "gstvkdevice.h"
 #include "gstvkdebug.h"
+#include "gstvkphysicaldevice-private.h"
 
 #include <string.h>
 
@@ -58,8 +59,8 @@ struct _GstVulkanDevicePrivate
   GPtrArray *enabled_extensions;
 
   gboolean opened;
-  guint queue_family_id;
-  guint n_queues;
+  GArray *queues;
+  GArray *queue_family_indices;
 
   GstVulkanFenceCache *fence_cache;
 };
@@ -176,12 +177,39 @@ static void
 gst_vulkan_device_constructed (GObject * object)
 {
   GstVulkanDevice *device = GST_VULKAN_DEVICE (object);
+  const char *optional_extensions[] = {
+    VK_KHR_SWAPCHAIN_EXTENSION_NAME,
+    VK_KHR_SAMPLER_YCBCR_CONVERSION_EXTENSION_NAME,
+#if defined(VK_KHR_timeline_semaphore)
+    VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME,
+#endif
+#if defined(VK_KHR_synchronization2)
+    VK_KHR_SYNCHRONIZATION_2_EXTENSION_NAME,
+#endif
+#if GST_VULKAN_HAVE_VIDEO_EXTENSIONS
+    VK_KHR_VIDEO_QUEUE_EXTENSION_NAME,
+    VK_KHR_VIDEO_DECODE_QUEUE_EXTENSION_NAME,
+    VK_KHR_VIDEO_DECODE_H264_EXTENSION_NAME,
+    VK_KHR_VIDEO_DECODE_H265_EXTENSION_NAME,
+    VK_KHR_VIDEO_ENCODE_QUEUE_EXTENSION_NAME,
+    VK_KHR_VIDEO_ENCODE_H264_EXTENSION_NAME,
+    VK_KHR_VIDEO_ENCODE_H265_EXTENSION_NAME,
+#if defined(VK_KHR_video_maintenance1)
+    VK_KHR_VIDEO_MAINTENANCE_1_EXTENSION_NAME,
+#endif
+#endif
+  };
+  int i;
 
   g_object_get (device->physical_device, "instance", &device->instance, NULL);
 
-  /* by default allow vkswapper to work for rendering to an output window.
-   * Ignore the failure if the extension does not exist. */
-  gst_vulkan_device_enable_extension (device, VK_KHR_SWAPCHAIN_EXTENSION_NAME);
+  /* Ignore the failure if the extension does not exist. */
+  for (i = 0; i < G_N_ELEMENTS (optional_extensions); i++) {
+    if (!gst_vulkan_device_enable_extension (device, optional_extensions[i])) {
+      GST_INFO_OBJECT (device, "Could not enable extension %s",
+          optional_extensions[i]);
+    }
+  }
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 }
@@ -214,6 +242,16 @@ gst_vulkan_device_dispose (GObject * object)
   GstVulkanDevice *device = GST_VULKAN_DEVICE (object);
   GstVulkanDevicePrivate *priv = GET_PRIV (device);
 
+  if (priv->queue_family_indices) {
+    g_array_unref (priv->queue_family_indices);
+    priv->queue_family_indices = NULL;
+  }
+
+  if (priv->queues) {
+    g_array_unref (priv->queues);
+    priv->queues = NULL;
+  }
+
   if (priv->fence_cache) {
     /* clear any outstanding fences */
     g_object_run_dispose (G_OBJECT (priv->fence_cache));
@@ -250,6 +288,128 @@ gst_vulkan_device_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/* https://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetParallel */
+/* TODO: add this function for general use and consider compiler builtins */
+static inline guint32
+_pop_count (guint32 n)
+{
+  n = n - ((n >> 1) & 0x55555555);
+  n = (n & 0x33333333) + ((n >> 2) & 0x33333333);
+  return (((n + (n >> 4)) & 0xF0F0F0F) * 0x1010101) >> 24;
+}
+
+/* look for the queue with more capabilities for the requested flag and also
+ * used by other flags, thus we could use the same queue for more ops. Though,
+ * perhaps it's not the best strategy for parallelism. */
+static inline int
+_pick_queue_family (VkQueueFamilyProperties * queue_family_props,
+    guint32 num_queue_families, VkQueueFlagBits flags, guint32 * family_scores)
+{
+  int i, index = -1;
+  guint32 score, max_score = 0;
+
+  for (i = 0; i < num_queue_families; i++) {
+    const VkQueueFlagBits queue_flags = queue_family_props[i].queueFlags;
+    if (queue_flags & flags) {
+      score = _pop_count (queue_flags) + family_scores[i];
+      if (score > max_score) {
+        index = i;
+        max_score = score;
+      }
+    }
+  }
+
+  if (index > -1)
+    family_scores[index]++;
+
+  return index;
+}
+
+static GArray *
+_append_queue_create_info (GArray * array, int family_index,
+    VkQueueFamilyProperties * queue_family_props)
+{
+  int i;
+  VkDeviceQueueCreateInfo queue_info;
+  gint queue_count;
+  gfloat *priorities;
+
+  if (family_index == -1)
+    return array;
+
+  for (i = 0; i < array->len; i++) {
+    VkDeviceQueueCreateInfo *qi =
+        &g_array_index (array, VkDeviceQueueCreateInfo, i);
+    if (qi->queueFamilyIndex == family_index)
+      return array;
+  }
+
+  /* shall we open all -- queue_family_props[family_index].queueCount ? */
+  queue_count = 1;
+
+  priorities = g_new (gfloat, queue_count);
+  for (i = 0; i < queue_count; i++)
+    priorities[i] = 1.0 / queue_count;
+
+  /* *INDENT-OFF* */
+  queue_info = (VkDeviceQueueCreateInfo) {
+      .sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO,
+      .queueFamilyIndex = family_index,
+      .queueCount = queue_count,
+      .pQueuePriorities = priorities,
+  };
+  /* *INDENT-ON* */
+
+  return g_array_append_val (array, queue_info);
+}
+
+
+/* Returns an array of VkDeviceQueueCreateInfo with the list of queues to
+ * create. The list will contain one or more queues which will support all the
+ * required families */
+static GArray *
+gst_vulkan_device_choose_queues (GstVulkanDevice * device)
+{
+  VkQueueFamilyProperties *queue_family_props;
+  GArray *array;
+  guint32 *family_scores, n_queue_families;
+  int graph_index, comp_index, tx_index;
+#if GST_VULKAN_HAVE_VIDEO_EXTENSIONS
+  int dec_index = -1;
+  int enc_index = -1;
+#endif
+
+  n_queue_families = device->physical_device->n_queue_families;
+  queue_family_props = device->physical_device->queue_family_props;
+
+  array = g_array_sized_new (FALSE, FALSE, sizeof (VkDeviceQueueCreateInfo),
+      n_queue_families);
+
+  family_scores = g_new0 (guint32, n_queue_families);
+
+  graph_index = _pick_queue_family (queue_family_props, n_queue_families,
+      VK_QUEUE_GRAPHICS_BIT, family_scores);
+  array = _append_queue_create_info (array, graph_index, queue_family_props);
+  comp_index = _pick_queue_family (queue_family_props, n_queue_families,
+      VK_QUEUE_COMPUTE_BIT, family_scores);
+  array = _append_queue_create_info (array, comp_index, queue_family_props);
+  tx_index = _pick_queue_family (queue_family_props, n_queue_families,
+      VK_QUEUE_TRANSFER_BIT, family_scores);
+  array = _append_queue_create_info (array, tx_index, queue_family_props);
+#if GST_VULKAN_HAVE_VIDEO_EXTENSIONS
+  dec_index = _pick_queue_family (queue_family_props, n_queue_families,
+      VK_QUEUE_VIDEO_DECODE_BIT_KHR, family_scores);
+  array = _append_queue_create_info (array, dec_index, queue_family_props);
+  enc_index = _pick_queue_family (queue_family_props, n_queue_families,
+      VK_QUEUE_VIDEO_ENCODE_BIT_KHR, family_scores);
+  array = _append_queue_create_info (array, enc_index, queue_family_props);
+#endif
+
+  g_free (family_scores);
+
+  return array;
+}
+
 /**
  * gst_vulkan_device_open:
  * @device: a #GstVulkanDevice
@@ -265,7 +425,6 @@ gboolean
 gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
 {
   GstVulkanDevicePrivate *priv = GET_PRIV (device);
-  VkPhysicalDevice gpu;
   VkResult err;
   guint i;
 
@@ -278,21 +437,14 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
     return TRUE;
   }
 
-  gpu = gst_vulkan_device_get_physical_device (device);
-
-  /* FIXME: allow overriding/selecting */
-  for (i = 0; i < device->physical_device->n_queue_families; i++) {
-    if (device->physical_device->
-        queue_family_props[i].queueFlags & VK_QUEUE_GRAPHICS_BIT)
-      break;
-  }
-  if (i >= device->physical_device->n_queue_families) {
+  priv->queues = gst_vulkan_device_choose_queues (device);
+  if (priv->queues->len == 0) {
+    g_array_unref (priv->queues);
+    priv->queues = NULL;
     g_set_error (error, GST_VULKAN_ERROR, VK_ERROR_INITIALIZATION_FAILED,
         "Failed to find a compatible queue family");
     goto error;
   }
-  priv->queue_family_id = i;
-  priv->n_queues = 1;
 
   GST_INFO_OBJECT (device, "Creating a device from physical %" GST_PTR_FORMAT
       " with %u layers and %u extensions", device->physical_device,
@@ -306,20 +458,15 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
         (gchar *) g_ptr_array_index (priv->enabled_extensions, i));
 
   {
-    VkDeviceQueueCreateInfo queue_info = { 0, };
+    VkPhysicalDevice gpu;
     VkDeviceCreateInfo device_info = { 0, };
-    gfloat queue_priority = 0.5;
-
-    queue_info.sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO;
-    queue_info.pNext = NULL;
-    queue_info.queueFamilyIndex = priv->queue_family_id;
-    queue_info.queueCount = priv->n_queues;
-    queue_info.pQueuePriorities = &queue_priority;
 
     device_info.sType = VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
-    device_info.pNext = NULL;
-    device_info.queueCreateInfoCount = 1;
-    device_info.pQueueCreateInfos = &queue_info;
+    device_info.pNext =
+        gst_vulkan_physical_device_get_features (device->physical_device);
+    device_info.queueCreateInfoCount = priv->queues->len;
+    device_info.pQueueCreateInfos = (VkDeviceQueueCreateInfo *)
+        priv->queues->data;
     device_info.enabledLayerCount = priv->enabled_layers->len;
     device_info.ppEnabledLayerNames =
         (const char *const *) priv->enabled_layers->pdata;
@@ -328,6 +475,7 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
         (const char *const *) priv->enabled_extensions->pdata;
     device_info.pEnabledFeatures = NULL;
 
+    gpu = gst_vulkan_device_get_physical_device (device);
     err = vkCreateDevice (gpu, &device_info, NULL, &device->device);
     if (gst_vulkan_error_to_g_error (err, error, "vkCreateDevice") < 0) {
       goto error;
@@ -338,6 +486,12 @@ gst_vulkan_device_open (GstVulkanDevice * device, GError ** error)
   /* avoid reference loops between us and the fence cache */
   gst_object_unref (device);
 
+  for (i = 0; i < priv->queues->len; i++) {
+    VkDeviceQueueCreateInfo *qi =
+        &g_array_index (priv->queues, VkDeviceQueueCreateInfo, i);
+    g_free ((gpointer) qi->pQueuePriorities);
+  }
+
   priv->opened = TRUE;
   GST_OBJECT_UNLOCK (device);
   return TRUE;
@@ -365,14 +519,20 @@ gst_vulkan_device_get_queue (GstVulkanDevice * device, guint32 queue_family,
 {
   GstVulkanDevicePrivate *priv = GET_PRIV (device);
   GstVulkanQueue *ret;
+  int i;
 
   g_return_val_if_fail (GST_IS_VULKAN_DEVICE (device), NULL);
   g_return_val_if_fail (device->device != NULL, NULL);
   g_return_val_if_fail (priv->opened, NULL);
-  g_return_val_if_fail (queue_family < priv->n_queues, NULL);
-  g_return_val_if_fail (queue_i <
-      device->physical_device->queue_family_props[queue_family].queueCount,
-      NULL);
+
+  for (i = 0; i < priv->queues->len; i++) {
+    VkDeviceQueueCreateInfo *qi =
+        &g_array_index (priv->queues, VkDeviceQueueCreateInfo, i);
+    if (qi->queueFamilyIndex == queue_family && qi->queueCount >= queue_i)
+      break;
+  }
+
+  g_return_val_if_fail (i < priv->queues->len, NULL);
 
   ret = g_object_new (GST_TYPE_VULKAN_QUEUE, NULL);
   gst_object_ref_sink (ret);
@@ -388,8 +548,9 @@ gst_vulkan_device_get_queue (GstVulkanDevice * device, guint32 queue_family,
 /**
  * gst_vulkan_device_foreach_queue:
  * @device: a #GstVulkanDevice
- * @func: (scope call): a #GstVulkanDeviceForEachQueueFunc to run for each #GstVulkanQueue
- * @user_data: (closure func): user data to pass to each call of @func
+ * @func: (scope call) (closure user_data): a #GstVulkanDeviceForEachQueueFunc
+ *    to run for each #GstVulkanQueue
+ * @user_data: user data to pass to each call of @func
  *
  * Iterate over each queue family available on #GstVulkanDevice
  *
@@ -401,22 +562,75 @@ gst_vulkan_device_foreach_queue (GstVulkanDevice * device,
 {
   GstVulkanDevicePrivate *priv = GET_PRIV (device);
   gboolean done = FALSE;
-  guint i;
+  guint i, j;
 
-  for (i = 0; i < priv->n_queues; i++) {
-    GstVulkanQueue *queue =
-        gst_vulkan_device_get_queue (device, priv->queue_family_id, i);
+  g_return_if_fail (GST_IS_VULKAN_DEVICE (device));
+  g_return_if_fail (priv->opened);
 
-    if (!func (device, queue, user_data))
-      done = TRUE;
+  for (i = 0; i < priv->queues->len; i++) {
+    VkDeviceQueueCreateInfo *qi =
+        &g_array_index (priv->queues, VkDeviceQueueCreateInfo, i);
 
-    gst_object_unref (queue);
+    for (j = 0; j < qi->queueCount; j++) {
+      GstVulkanQueue *queue =
+          gst_vulkan_device_get_queue (device, qi->queueFamilyIndex, j);
 
-    if (done)
-      break;
+      if (!func (device, queue, user_data))
+        done = TRUE;
+
+      gst_object_unref (queue);
+
+      if (done)
+        return;
+    }
   }
 }
 
+/**
+ * gst_vulkan_device_queue_family_indices:
+ * @device: a #GstVulkanDevice
+ *
+ * Returns: (element-type uint32_t) (transfer full): An array with the family
+ *     indexes of the created queues in @device
+ *
+ * Since: 1.24
+ */
+GArray *
+gst_vulkan_device_queue_family_indices (GstVulkanDevice * device)
+{
+  GstVulkanDevicePrivate *priv = GET_PRIV (device);
+  guint i, j;
+
+  g_return_val_if_fail (GST_IS_VULKAN_DEVICE (device), NULL);
+  g_return_val_if_fail (priv->opened, NULL);
+
+  GST_OBJECT_LOCK (device);
+
+  if (priv->queue_family_indices)
+    goto beach;
+
+  priv->queue_family_indices =
+      g_array_sized_new (FALSE, FALSE, sizeof (uint32_t), priv->queues->len);
+
+  for (i = 0; i < priv->queues->len; i++) {
+    VkDeviceQueueCreateInfo *qi =
+        &g_array_index (priv->queues, VkDeviceQueueCreateInfo, i);
+
+    for (j = 0; j < priv->queue_family_indices->len; j++) {
+      uint32_t qfi = g_array_index (priv->queue_family_indices, uint32_t, j);
+      if (qfi == qi->queueFamilyIndex)
+        break;
+    }
+
+    if (j == priv->queue_family_indices->len)
+      g_array_append_val (priv->queue_family_indices, qi->queueFamilyIndex);
+  }
+
+beach:
+  GST_OBJECT_UNLOCK (device);
+  return g_array_ref (priv->queue_family_indices);
+}
+
 /**
  * gst_vulkan_device_get_proc_address:
  * @device: a #GstVulkanDevice
@@ -476,7 +690,7 @@ gst_vulkan_device_get_physical_device (GstVulkanDevice * device)
 /**
  * gst_context_set_vulkan_device:
  * @context: a #GstContext
- * @device: a #GstVulkanDevice
+ * @device: (transfer none) (nullable): a #GstVulkanDevice
  *
  * Sets @device on @context
  *
@@ -503,7 +717,7 @@ gst_context_set_vulkan_device (GstContext * context, GstVulkanDevice * device)
 /**
  * gst_context_get_vulkan_device:
  * @context: a #GstContext
- * @device: resulting #GstVulkanDevice
+ * @device: (out) (optional) (nullable) (transfer full): resulting #GstVulkanDevice
  *
  * Returns: Whether @device was in @context
  *
@@ -644,32 +858,14 @@ gst_vulkan_device_create_fence (GstVulkanDevice * device, GError ** error)
   return gst_vulkan_fence_cache_acquire (priv->fence_cache, error);
 }
 
-/* reimplement a specfic case of g_ptr_array_find_with_equal_func as that
- * requires Glib 2.54 */
-static gboolean
-ptr_array_find_string (GPtrArray * array, const gchar * str, guint * index)
-{
-  guint i;
-
-  for (i = 0; i < array->len; i++) {
-    gchar *val = (gchar *) g_ptr_array_index (array, i);
-    if (g_strcmp0 (val, str) == 0) {
-      if (index)
-        *index = i;
-      return TRUE;
-    }
-  }
-
-  return FALSE;
-}
-
 static gboolean
 gst_vulkan_device_is_extension_enabled_unlocked (GstVulkanDevice * device,
     const gchar * name, guint * index)
 {
   GstVulkanDevicePrivate *priv = GET_PRIV (device);
 
-  return ptr_array_find_string (priv->enabled_extensions, name, index);
+  return g_ptr_array_find_with_equal_func (priv->enabled_extensions, name,
+      g_str_equal, index);
 }
 
 /**
@@ -798,7 +994,8 @@ gst_vulkan_device_is_layer_enabled_unlocked (GstVulkanDevice * device,
 {
   GstVulkanDevicePrivate *priv = GET_PRIV (device);
 
-  return ptr_array_find_string (priv->enabled_layers, name, NULL);
+  return g_ptr_array_find_with_equal_func (priv->enabled_layers, name,
+      g_str_equal, NULL);
 }
 
 /**
@@ -871,3 +1068,58 @@ gst_vulkan_device_enable_layer (GstVulkanDevice * device, const gchar * name)
 
   return ret;
 }
+
+struct choose_queue
+{
+  guint expected_flags;
+  GstVulkanQueue *queue;
+};
+
+static gboolean
+_choose_queue (GstVulkanDevice * device, GstVulkanQueue * queue,
+    struct choose_queue *data)
+{
+  guint flags =
+      device->physical_device->queue_family_props[queue->family].queueFlags;
+
+  if ((flags & data->expected_flags) != 0) {
+    if (data->queue)
+      gst_object_unref (data->queue);
+    data->queue = gst_object_ref (queue);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+/**
+ * gst_vulkan_device_select_queue
+ * @device: a #GstVulkanDevice
+ * @expected_flags: a VkQueueFlagBits
+ *
+ * Select a compatible queue from the @device supporting the @expected_flags.
+ *
+ * Returns: (transfer full) (nullable): a #GstVulkanQueue for @queue matching
+ *                                      the @expected_flags
+ *
+ * Since: 1.24
+ */
+GstVulkanQueue *
+gst_vulkan_device_select_queue (GstVulkanDevice * device,
+    VkQueueFlagBits expected_flags)
+{
+  struct choose_queue data;
+
+  data.expected_flags = expected_flags;
+  data.queue = NULL;
+
+  if (!gst_vulkan_device_open (device, NULL)) {
+    gst_object_unref (device);
+    goto beach;
+  }
+  gst_vulkan_device_foreach_queue (device,
+      (GstVulkanDeviceForEachQueueFunc) _choose_queue, &data);
+
+beach:
+  return data.queue;
+}
